@@ -6,10 +6,16 @@ extern crate proc_macro;
 
 use proc_macro::TokenStream;
 use proc_macro2::TokenStream as TokenStream2;
-use std::{env, path::Path};
-use syn::{Data, DeriveInput, Fields, Lit, Meta, MetaNameValue};
+use std::{
+  env,
+  iter::FromIterator,
+  path::{Path, PathBuf},
+};
+use syn::{Data, DeriveInput, Expr, ExprLit, Fields, Lit, Meta, MetaNameValue};
 
-fn embedded(ident: &syn::Ident, folder_path: String, prefix: Option<&str>, includes: &[String], excludes: &[String]) -> TokenStream2 {
+fn embedded(
+  ident: &syn::Ident, relative_folder_path: Option<&str>, absolute_folder_path: String, prefix: Option<&str>, includes: &[String], excludes: &[String],
+) -> TokenStream2 {
   extern crate rust_embed_utils;
 
   let mut match_values = Vec::<TokenStream2>::new();
@@ -17,8 +23,8 @@ fn embedded(ident: &syn::Ident, folder_path: String, prefix: Option<&str>, inclu
 
   let includes: Vec<&str> = includes.iter().map(AsRef::as_ref).collect();
   let excludes: Vec<&str> = excludes.iter().map(AsRef::as_ref).collect();
-  for rust_embed_utils::FileEntry { rel_path, full_canonical_path } in rust_embed_utils::get_files(folder_path, &includes, &excludes) {
-    match_values.push(embed_file(&rel_path, &full_canonical_path));
+  for rust_embed_utils::FileEntry { rel_path, full_canonical_path } in rust_embed_utils::get_files(absolute_folder_path.clone(), &includes, &excludes) {
+    match_values.push(embed_file(relative_folder_path.clone(), &rel_path, &full_canonical_path));
 
     list_values.push(if let Some(prefix) = prefix {
       format!("{}{}", prefix, rel_path)
@@ -153,13 +159,22 @@ fn dynamic(ident: &syn::Ident, folder_path: String, prefix: Option<&str>, includ
   }
 }
 
-fn generate_assets(ident: &syn::Ident, folder_path: String, prefix: Option<String>, includes: Vec<String>, excludes: Vec<String>) -> TokenStream2 {
-  let embedded_impl = embedded(ident, folder_path.clone(), prefix.as_deref(), &includes, &excludes);
+fn generate_assets(
+  ident: &syn::Ident, relative_folder_path: Option<&str>, absolute_folder_path: String, prefix: Option<String>, includes: Vec<String>, excludes: Vec<String>,
+) -> TokenStream2 {
+  let embedded_impl = embedded(
+    ident,
+    relative_folder_path,
+    absolute_folder_path.clone(),
+    prefix.as_deref(),
+    &includes,
+    &excludes,
+  );
   if cfg!(feature = "debug-embed") {
     return embedded_impl;
   }
 
-  let dynamic_impl = dynamic(ident, folder_path, prefix.as_deref(), &includes, &excludes);
+  let dynamic_impl = dynamic(ident, absolute_folder_path, prefix.as_deref(), &includes, &excludes);
 
   quote! {
       #embedded_impl
@@ -167,17 +182,27 @@ fn generate_assets(ident: &syn::Ident, folder_path: String, prefix: Option<Strin
   }
 }
 
-fn embed_file(rel_path: &str, full_canonical_path: &str) -> TokenStream2 {
+fn embed_file(folder_path: Option<&str>, rel_path: &str, full_canonical_path: &str) -> TokenStream2 {
   let file = rust_embed_utils::read_file_from_fs(Path::new(full_canonical_path)).expect("File should be readable");
   let hash = file.metadata.sha256_hash();
   let last_modified = match file.metadata.last_modified() {
     Some(last_modified) => quote! { Some(#last_modified) },
     None => quote! { None },
   };
+  #[cfg(feature = "mime-guess")]
+  let mimetype_tokens = {
+    let mt = file.metadata.mimetype();
+    quote! { , #mt }
+  };
+  #[cfg(not(feature = "mime-guess"))]
+  let mimetype_tokens = TokenStream2::new();
 
   let embedding_code = if cfg!(feature = "compression") {
+    // Print some debugging information
+    let full_relative_path = PathBuf::from_iter([folder_path.expect("folder_path must be provided under `compression` feature"), rel_path]);
+    let full_relative_path = full_relative_path.to_string_lossy();
     quote! {
-      rust_embed::flate!(static FILE: [u8] from #full_canonical_path);
+      rust_embed::flate!(static FILE: [u8] from #full_relative_path);
       let bytes = &FILE[..];
     }
   } else {
@@ -192,7 +217,7 @@ fn embed_file(rel_path: &str, full_canonical_path: &str) -> TokenStream2 {
 
           Some(rust_embed::EmbeddedFile {
               data: std::borrow::Cow::from(bytes),
-              metadata: rust_embed::Metadata::__rust_embed_new([#(#hash),*], #last_modified)
+              metadata: rust_embed::Metadata::__rust_embed_new([#(#hash),*], #last_modified #mimetype_tokens)
           })
       }
   }
@@ -203,10 +228,12 @@ fn find_attribute_values(ast: &syn::DeriveInput, attr_name: &str) -> Vec<String>
   ast
     .attrs
     .iter()
-    .filter(|value| value.path.is_ident(attr_name))
-    .filter_map(|attr| attr.parse_meta().ok())
-    .filter_map(|meta| match meta {
-      Meta::NameValue(MetaNameValue { lit: Lit::Str(val), .. }) => Some(val.value()),
+    .filter(|value| value.path().is_ident(attr_name))
+    .filter_map(|attr| match &attr.meta {
+      Meta::NameValue(MetaNameValue {
+        value: Expr::Lit(ExprLit { lit: Lit::Str(val), .. }),
+        ..
+      }) => Some(val.value()),
       _ => None,
     })
     .collect()
@@ -240,26 +267,30 @@ fn impl_rust_embed(ast: &syn::DeriveInput) -> TokenStream2 {
   let folder_path = shellexpand::full(&folder_path).unwrap().to_string();
 
   // Base relative paths on the Cargo.toml location
-  let folder_path = if Path::new(&folder_path).is_relative() {
-    Path::new(&env::var("CARGO_MANIFEST_DIR").unwrap())
-      .join(folder_path)
+  let (relative_path, absolute_folder_path) = if Path::new(&folder_path).is_relative() {
+    let absolute_path = Path::new(&env::var("CARGO_MANIFEST_DIR").unwrap())
+      .join(&folder_path)
       .to_str()
       .unwrap()
-      .to_owned()
+      .to_owned();
+    (Some(folder_path.clone()), absolute_path)
   } else {
-    folder_path
+    if cfg!(feature = "compression") {
+      panic!("`folder` must be a relative path under `compression` feature.")
+    }
+    (None, folder_path)
   };
 
-  if !Path::new(&folder_path).exists() {
+  if !Path::new(&absolute_folder_path).exists() {
     let mut message = format!(
       "#[derive(RustEmbed)] folder '{}' does not exist. cwd: '{}'",
-      folder_path,
+      absolute_folder_path,
       std::env::current_dir().unwrap().to_str().unwrap()
     );
 
     // Add a message about the interpolate-folder-path feature if the path may
     // include a variable
-    if folder_path.contains('$') && cfg!(not(feature = "interpolate-folder-path")) {
+    if absolute_folder_path.contains('$') && cfg!(not(feature = "interpolate-folder-path")) {
       message += "\nA variable has been detected. RustEmbed can expand variables \
                   when the `interpolate-folder-path` feature is enabled.";
     }
@@ -267,7 +298,7 @@ fn impl_rust_embed(ast: &syn::DeriveInput) -> TokenStream2 {
     panic!("{}", message);
   };
 
-  generate_assets(&ast.ident, folder_path, prefix, includes, excludes)
+  generate_assets(&ast.ident, relative_path.as_deref(), absolute_folder_path, prefix, includes, excludes)
 }
 
 #[proc_macro_derive(RustEmbed, attributes(folder, prefix, include, exclude))]
